package stocks.client;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.StringTokenizer;

import org.omg.CORBA.ORB;

public class Client {
  ClientQuoterFactory _clientFactory = null;
  ORB _orb = null;
  HashMap _mapQuoters = null;

  Client() {
    _mapQuoters = new HashMap();
  }
 
  /**
   * @param args
   */
  public static void main(String[] args) {
    try {
      Client client = new Client();
      client.run( args );
    }
    catch( Throwable e ) {
      e.printStackTrace();
    }
  }
  
  protected void run( String[] args ){
    _orb = ORB.init(args,null);
    _clientFactory = new ClientQuoterFactoryImpl(_orb);
    
    run_cmd_loop();
    
    _orb = null;
  }
  
  private void run_cmd_loop() {

    BufferedReader input =                         
      new BufferedReader(new InputStreamReader(System.in));
    String sShell = "\n> ";
    String sCommand = "";
    
    while( !sCommand.equals("quit") )
    {
      System.out.print(sShell);

      try {
        sCommand = input.readLine();
      }
      catch( IOException e ) {
        continue;
      }
      
      StringTokenizer st = new StringTokenizer(sCommand);
      if( !st.hasMoreTokens() ) {
        print_help();
        continue;
      }
      
      sCommand = st.nextToken();
      if( sCommand.equals("q") || sCommand.equals("quoter") ) { 
        quoter( st );
      }
      else {
        print_help();
      }
    }
  }

  private void quoter(  StringTokenizer st ) {
    if( !st.hasMoreTokens() )
    {
      System.out.println("Parameter name is missing.");
      return;
    }
    String name = st.nextToken();

    if( !st.hasMoreTokens() )
    {
      System.out.println("Parameter operation is missing.");
      return;
    }
    String operation = st.nextToken();

    if( operation.equals("c") || operation.equals("create") )
    {
      quoter_create( name );
    }
    else {
      if( !_mapQuoters.containsKey(name) )
      {
        System.out.println("Quoter name not found.");
        return;
      }
      ClientStockQuoter quoter = (ClientStockQuoter)_mapQuoters.get(name);
      
      if( operation.equals("gn") || operation.equals("getname") )
      {
        quoter_getname( quoter );
      }
      else if( operation.equals("gs") || operation.equals("getstock") )
      {
        quoter_getstock( quoter, st );
      }
    }
  }

  private void quoter_create( String name ) {
    try {
      ClientStockQuoter quoter = _clientFactory.create_quoter(name);
      _mapQuoters.put(name, quoter);
    }
    catch( InvalidQuoterException e ) {
      System.out.println("Invalid quoter name given.");
    }
  }
  
  private void quoter_getname( ClientStockQuoter quoter ) {
    System.out.println("Quoter name: " + quoter.getQuoterName());
  }

  private void quoter_getstock( ClientStockQuoter quoter, StringTokenizer st ) {
    if( !st.hasMoreTokens() )
    {
      System.out.println("Parameter stock-name is missing.");
      return;
    }
    String name = st.nextToken();
    ClientStock sStock = null;

    try {
      sStock = quoter.getStock(name);
    }
    catch( InvalidStockException e ) {
      System.out.println("Stock " + name + " not found.");
      return;
    }
    
    stock( sStock, st );
  }
  
  private void stock( ClientStock sStock, StringTokenizer st ) {
    if( !st.hasMoreTokens() )
    {
      System.out.println("Parameter stock-operation is missing.");
      return;
    }
    String operation = st.nextToken();
    
    if( operation.equals("gn" ) || operation.equals("getname") ) {
      System.out.println("Stock name: " + sStock.getName());
    }
    else if( operation.equals("gs" ) || operation.equals("getsymbol") ) {
      System.out.println("Stock symbol: " + sStock.getSymbol());
    }
    else if( operation.equals("gi" ) || operation.equals("getisin") ) {
      System.out.println("Stock ISIN: " + sStock.getISIN());
    }
    else if( operation.equals("gt" ) || operation.equals("gettype") ) {
      System.out.println("Stock type: " + sStock.getStockType());
    }
    else if( operation.equals("high" ) || operation.equals("gethigh") ) {
      System.out.println("High: " + sStock.getHigh());
    }
    else if( operation.equals("low" ) || operation.equals("getlow") ) {
      System.out.println("Low: " + sStock.getLow());
    }
    else if( operation.equals("last" ) || operation.equals("getlast") ) {
      System.out.println("Last: " + sStock.getLast());
    }
  }

  private void print_help() {
    System.out.println("q(uoter) <name> c(reate)");
    System.out.println("                g(et)n(ame)");
    System.out.println("                g(et)s(tock) <name> g(et)n(ame)");
    System.out.println("                                    g(et)s(ymbol)");
    System.out.println("                                    g(et)i(sin)");
    System.out.println("                                    g(et)t(ype)");
    System.out.println("                                    (get)high");
    System.out.println("                                    (get)low");
    System.out.println("                                    (get)last");
  }
}
