package stocks.client;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;

import org.omg.CORBA.ORB;

import stocks.QuoterFactoryHelper;
import stocks.QuoterFactory;
import stocks.InvalidQuoter;
import stocks.Quoter;
import stocks.StockQuoter;
import stocks.StockQuoterHelper;
import stocks.StockInfoPackage.StockQuoteHelper;

public class ClientQuoterFactoryImpl implements ClientQuoterFactory {
  private QuoterFactory _serverFactory = null;

  public ClientQuoterFactoryImpl( ORB orb )
	{
    String ior = "";
    String filename = "quoter_factory.ior";

    try {
      FileInputStream fis = new FileInputStream(filename);
      BufferedReader d = new BufferedReader(new InputStreamReader(fis));
      ior = d.readLine();
    }
    catch( FileNotFoundException e ) {
      System.out.println("Could not find file " + filename + ".");
      System.exit(1);
    }
    catch( IOException e ) {
      System.out.println("Could not read ior from file.");
      System.exit(1);
    }
    
    org.omg.CORBA.Object obj = orb.string_to_object(ior);
    _serverFactory = QuoterFactoryHelper.narrow(obj);
  }
  
  public ClientStockQuoter create_quoter(String name)
      throws InvalidQuoterException {
    Quoter quoter = null;
    
    try {
      quoter = _serverFactory.create_quoter(name);
    }
    catch( InvalidQuoter e ) {
      throw new InvalidQuoterException();
    }
    if( null == quoter )
      throw new InvalidQuoterException("Server returned null");
    //Quoter quoter = (Quoter)quoter;
    StockQuoter stockQuoter = StockQuoterHelper.narrow(quoter);
    
    return( new ClientStockQuoterImpl(stockQuoter) );
  }

}
